import * as fs from 'fs-extra';
import * as crypto from 'crypto';
import * as glob from 'glob';
import * as path from 'path';
import { promisify } from './Promisify';
var NodeEnvironment = (function () {
    function NodeEnvironment(tempPath, log) {
        this.filePath = tempPath + '/cache.json';
        this.tempPath = tempPath;
        this.data = {};
        this.log = log;
        this.cacheMissesCurrentlyInProgress = {};
    }
    NodeEnvironment.prototype.calculateHash = function (filePath) {
        return fs.stat(filePath)
            .then(function (stat) {
            // stat.mtime.getTime() is not in here because it causes problems with git
            var identifier = path + ' ' + stat.size;
            return crypto.createHash('sha1').update(identifier).digest('hex');
        });
    };
    NodeEnvironment.prototype.createHash = function (string) {
        var hash = crypto.createHash('sha1');
        hash.update(string);
        return hash.digest('hex');
    };
    NodeEnvironment.prototype.lookup = function (type, key, cacheMissFunction, version) {
        var _this = this;
        version = version || 0;
        if (!this.data[type]) {
            this.data[type] = {};
        }
        var item = this.data[type][key];
        if (item && item.version === version) {
            return Promise.resolve(item.data);
        }
        // This bit of code avoids a race condition between two lookups for the same type/key
        // Without this cacheMissFunction would be called twice
        this.cacheMissesCurrentlyInProgress[type] = this.cacheMissesCurrentlyInProgress[type] || {};
        if (!this.cacheMissesCurrentlyInProgress[type][key]) {
            this.cacheMissesCurrentlyInProgress[type][key] = Promise.resolve()
                .then(function () { return cacheMissFunction(); })
                .then(function (result) {
                delete _this.cacheMissesCurrentlyInProgress[type][key];
                return result;
            });
        }
        return this.cacheMissesCurrentlyInProgress[type][key]
            .then(function (result) {
            // Make sure we're only using serialisable results
            result = JSON.parse(JSON.stringify(result));
            _this.data[type][key] = {
                version: version,
                data: result
            };
            return result;
        });
    };
    NodeEnvironment.prototype.getCachePath = function (basename) {
        return path.join(this.tempPath, basename);
    };
    NodeEnvironment.prototype.save = function () {
        return promisify(fs.writeFile)(this.filePath, JSON.stringify(this.data, null, 4));
    };
    NodeEnvironment.prototype.clean = function () {
        return fs.remove(this.tempPath);
    };
    NodeEnvironment.prototype.load = function () {
        var _this = this;
        return fs.ensureDir(this.tempPath)
            .then(function () {
            return fs.readFile(_this.filePath, 'utf8')
                .then(function (raw) { return _this.data = JSON.parse(raw); })
                .catch(function () {
                // This failure is recoverable (and even expected in case of a cold cache)
                _this.log.info('Using fresh cache for %s', _this.filePath);
            });
        });
    };
    NodeEnvironment.prototype.cleanPath = function (dir) {
        return fs.remove(dir);
    };
    NodeEnvironment.prototype.copyTo = function (targetFilePath, toPath) {
        return fs.copy(targetFilePath, toPath);
    };
    NodeEnvironment.prototype.joinPaths = function () {
        var paths = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            paths[_i] = arguments[_i];
        }
        return path.join.apply(path, paths);
    };
    NodeEnvironment.prototype.ensureDir = function (dir) {
        return fs.ensureDir(dir);
    };
    NodeEnvironment.prototype.glob = function (pattern) {
        return promisify(glob)(pattern);
    };
    NodeEnvironment.prototype.writeFile = function (outputFilePath, raw) {
        return promisify(fs.writeFile)(outputFilePath, raw);
    };
    return NodeEnvironment;
}());
export default NodeEnvironment;
