import * as fs from 'fs-extra';
import * as crypto from 'crypto';
import * as glob from 'glob';
import * as path from 'path';
import {Logger, Dictionary, IEnvironment} from '@image/packer';
import {promisify} from './Promisify';

export default class NodeEnvironment implements IEnvironment {
  private filePath: string;
  private tempPath: string;
  private data: Dictionary<Dictionary<{
    version: any,
    data: any
  }>>;
  private log: Logger;
  private cacheMissesCurrentlyInProgress: Dictionary<Dictionary<Promise<any>>>;

  constructor(tempPath: string, log: any) {
    this.filePath = tempPath + '/cache.json';
    this.tempPath = tempPath;
    this.data = {};
    this.log = log;
    this.cacheMissesCurrentlyInProgress = {};
  }

  public calculateHash(filePath: string): Promise<string> {
    return fs.stat(filePath)
      .then(stat => {
        // stat.mtime.getTime() is not in here because it causes problems with git
        const identifier = path + ' ' + stat.size;
        return crypto.createHash('sha1').update(identifier).digest('hex');
      });
  }

  public createHash(string: any): string {
    const hash = crypto.createHash('sha1');
    hash.update(string);
    return hash.digest('hex');
  }

  public lookup(type: string, key: string, cacheMissFunction: () => any, version?: any) {
    version = version || 0;

    if (!this.data[type]) {
      this.data[type] = {};
    }

    const item = this.data[type][key];
    if (item && item.version === version) {
      return Promise.resolve(item.data);
    }

    // This bit of code avoids a race condition between two lookups for the same type/key
    // Without this cacheMissFunction would be called twice
    this.cacheMissesCurrentlyInProgress[type] = this.cacheMissesCurrentlyInProgress[type] || {};
    if (!this.cacheMissesCurrentlyInProgress[type][key]) {
      this.cacheMissesCurrentlyInProgress[type][key] = Promise.resolve()
        .then(() => cacheMissFunction())
        .then((result) => {
          delete this.cacheMissesCurrentlyInProgress[type][key];
          return result;
        });
    }

    return this.cacheMissesCurrentlyInProgress[type][key]
      .then((result) => {
        // Make sure we're only using serialisable results
        result = JSON.parse(JSON.stringify(result));
        this.data[type][key] = {
          version: version,
          data: result
        };
        return result;
      });
  }

  public getCachePath(basename: string): string {
    return path.join(this.tempPath, basename);
  }


  public save(): Promise<void> {
    return promisify<string, string, void>(fs.writeFile)(this.filePath, JSON.stringify(this.data, null, 4));
  }

  public clean(): Promise<void> {
    return fs.remove(this.tempPath);
  }

  public load(): Promise<void> {
    return fs.ensureDir(this.tempPath)
      .then(() => {
        return fs.readFile(this.filePath, 'utf8')
          .then(raw => this.data = JSON.parse(raw))
          .catch(() => {
            // This failure is recoverable (and even expected in case of a cold cache)
            this.log.info('Using fresh cache for %s', this.filePath);
          })
      });
  }

  public cleanPath(dir: string): Promise<void> {
    return fs.remove(dir);
  }

  public copyTo(targetFilePath: string, toPath: string): Promise<void> {
    return fs.copy(targetFilePath, toPath);
  }

  public joinPaths(...paths: string[]): string {
    return path.join(...paths);
  }

  public ensureDir(dir: string): Promise<void> {
    return fs.ensureDir(dir)
  }

  public glob(pattern: string): Promise<string[]> {
    return promisify<string, string[]>(glob)(pattern);
  }

  public writeFile(outputFilePath: string, raw: string): Promise<void> {
    return promisify<string, string, void>(fs.writeFile)(outputFilePath, raw);
  }
}
